package cn.com.qmth.mps.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;

import cn.com.qmth.mps.bean.User;
import cn.com.qmth.mps.service.AuthService;
import cn.com.qmth.mps.vo.AdminLoginVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "登录相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/auth")
public class AuthController extends BaseController {
	@Autowired
	private AuthService authService;

	@ApiOperation(value = "管理端登录")
	@PostMapping("login/admin")
	public AdminLoginVo loginAdmin(@RequestParam String loginName, @RequestParam String password) {
		return authService.loginAdmin(loginName,password);
	}

	@ApiOperation(value = "微信小程序登录")
	@PostMapping("login/wxapp/login-code")
	public AdminLoginVo loginWxApp(@RequestParam String loginCode) {
		return authService.loginWxAppCode(loginCode);
	}
	
	@ApiOperation(value = "绑定微信手机号并登录（新api）")
	@PostMapping("login/wxapp/phone-code")
	public AdminLoginVo loginWxAppByPhoneCode(@RequestParam String loginCode,@RequestParam String phoneCode) {
		return authService.loginWxAppByPhoneCode(loginCode,phoneCode);
	}
	
	@ApiOperation(value = "绑定微信手机号并登录（旧api）")
	@PostMapping("login/wxapp/encrypted-data")
	public AdminLoginVo loginWxAppByEncryptedData(@RequestParam String loginCode,@RequestParam String encryptedData,@RequestParam String iv) {
		return authService.loginWxAppByEncryptedData(loginCode,encryptedData,iv);
	}

	@Aac(strict = BOOL.FALSE, auth = BOOL.TRUE)
	@ApiOperation(value = "登出")
	@PostMapping("logout")
	public void logout() {
		User user = getAccessUser();
        authService.logout(user);
	}

}