package cn.com.qmth.mps.service.impl;

import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.mps.bean.User;
import cn.com.qmth.mps.dao.SchoolDao;
import cn.com.qmth.mps.entity.SchoolEntity;
import cn.com.qmth.mps.enums.Role;
import cn.com.qmth.mps.service.SchoolService;
import cn.com.qmth.mps.util.PageUtil;
import cn.com.qmth.mps.util.SolarHttpUtil;
import cn.com.qmth.mps.vo.school.SchoolDomain;
import cn.com.qmth.mps.vo.school.SchoolQuery;
import cn.com.qmth.mps.vo.school.SchoolVo;

@Service
public class SchoolServiceImpl extends ServiceImpl<SchoolDao, SchoolEntity> implements SchoolService {
	@Value("${qmth.solar.host}")
	private String host;
	@Value("${qmth.solar.org.uri}")
	private String uri;
	@Value("${access-key}")
	private String accessKey;
	@Value("${access-secret}")
	private String accessSecret;
	@Value("${wxapp-url}")
	private String wxappUrl;

	@Transactional
	@Override
	public void syncSchool(User user) {
		if (!user.getRole().equals(Role.SUPER_ADMIN)) {
			throw new StatusException("没有权限");
		}
		SolarHttpUtil orgUtil = new SolarHttpUtil(accessKey, accessSecret, host, uri);
		List<SchoolEntity> list = orgUtil.getOrgs();
		if (CollectionUtils.isEmpty(list)) {
			return;
		}
		for (SchoolEntity org : list) {
			addSchoolForSync(user, org);
		}
	}

	private void addSchoolForSync(User user, SchoolEntity school) {
		SchoolEntity old = this.findSchoolByCode(school.getCode());
		if (old != null) {
			old.setAccessKey(school.getAccessKey());
			old.setAccessSecret(school.getAccessSecret());
			this.updateById(old);
		} else {
			school.setId(null);
			school.setEnable(true);
			this.save(school);
		}
	}

	private SchoolEntity findSchoolByCode(String code) {
		QueryWrapper<SchoolEntity> wrapper = new QueryWrapper<>();
		LambdaQueryWrapper<SchoolEntity> lw = wrapper.lambda();
		lw.eq(SchoolEntity::getCode, code);
		return this.getOne(wrapper);
	}

	@Transactional
	@Override
	public void saveSchool(User user, SchoolDomain domain) {
		if (!user.getRole().equals(Role.SUPER_ADMIN) && domain.getId() == null) {
			throw new StatusException("没有权限");
		}
		if (!user.getRole().equals(Role.SUPER_ADMIN) && domain.getId() != null
				&& !user.getSchoolId().equals(domain.getId())) {
			throw new StatusException("没有权限");
		}
		if (domain.getId() == null && StringUtils.isBlank(domain.getCode())) {
			throw new StatusException("学校编码不能为空");
		}
		if (StringUtils.isBlank(domain.getName())) {
			throw new StatusException("学校名称不能为空");
		}
		if (domain.getEnable()==null) {
			throw new StatusException("启用禁用不能为空");
		}
		SchoolEntity ue = null;
		if (domain.getId() != null) {
			ue = this.getById(domain.getId());
			if (ue == null) {
				throw new StatusException("未找到学校");
			}
		} else {
			if (findSchoolByCode(domain.getCode()) != null) {
				throw new StatusException("学校编码已存在");
			}
			ue = new SchoolEntity();
			ue.setCode(domain.getCode());
		}
		ue.setEnable(domain.getEnable());
		ue.setName(domain.getName());
		ue.setContacts(domain.getContacts());
		ue.setTelephone(domain.getTelephone());
		ue.setRegion(domain.getRegion());
		this.saveOrUpdate(ue);
	}

	@Override
	public PageResult<SchoolVo> page(SchoolQuery query, User user) {
		Long schoolId = null;
		if (!user.getRole().equals(Role.SUPER_ADMIN)) {
			schoolId = user.getSchoolId();
		}
		IPage<SchoolVo> iPage = this.baseMapper.page(new Page<SchoolVo>(query.getPageNumber(), query.getPageSize()),
				query, schoolId);
		if (CollectionUtils.isNotEmpty(iPage.getRecords())) {
			for (SchoolVo vo : iPage.getRecords()) {
				vo.setQrCode(wxappUrl);
			}
		}
		return PageUtil.of(iPage);
	}

	@Override
	public SchoolVo info(Long id, User user) {
		if (!user.getRole().equals(Role.SUPER_ADMIN) && !user.getSchoolId().equals(id)) {
			return null;
		}
		SchoolEntity ue = this.getById(id);
		if (ue == null) {
			throw new StatusException("未找到学校");
		}
		SchoolVo vo = new SchoolVo();
		BeanUtils.copyProperties(ue, vo);
		vo.setQrCode(wxappUrl);
		return vo;
	}

	@Transactional
	@Override
	public void toggle(List<Long> ids, Boolean enable, User user) {
		if (!user.getRole().equals(Role.SUPER_ADMIN)) {
			throw new StatusException("没有权限");
		}
		if (ids.contains(1L)) {
			throw new StatusException("默认学校不能禁用（ID：1）");
		}
		UpdateWrapper<SchoolEntity> wrapper = new UpdateWrapper<>();
		LambdaUpdateWrapper<SchoolEntity> lw = wrapper.lambda();
		lw.set(SchoolEntity::getEnable, enable);
		lw.in(SchoolEntity::getId, ids);
		this.update(wrapper);
	}
}
