package cn.com.qmth.mps.service.impl;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.mps.bean.User;
import cn.com.qmth.mps.config.SysProperty;
import cn.com.qmth.mps.service.SessionService;

@Service
public class SessionServiceImpl implements SessionService {
	private final static Map<String, User> userSessionData = new ConcurrentHashMap<>();

	@Autowired
	private SysProperty sysProperty;

	@Override
	public Long updateUserSession(User user) {
		if (user.getSessionId() == null) {
			return null;
		}
		user.setActiveTime(new Date().getTime());
		userSessionData.put(user.getSessionId(), user);
		return user.getActiveTime();
	}

	@Override
	public void clearTimeOutUser() {
		Date d = new Date();
		Long now = d.getTime();
		for (String k : userSessionData.keySet()) {
			if (now - userSessionData.get(k).getActiveTime() > getTimeout()) {
				userSessionData.remove(k);
			}
		}
	}

	@Override
	public void userLogout(User user) {
		userSessionData.remove(user.getSessionId());
	}

	@Override
	public void userLogin(User user) {
		updateUserSession(user);
	}

	@Override
	public User getSessionUser(String sessionId) {
		return userSessionData.get(sessionId);
	}

	@Override
	public int getOnlineCount() {
		return userSessionData.size();
	}

	private long getTimeout() {
		return (sysProperty.getSessionTimeout() == null ? 7200 : sysProperty.getSessionTimeout()) * 1000L;
	}

}
