package cn.com.qmth.mps.util;

import java.util.ArrayList;
import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.core.collection.PageResult;

public class PageUtil {
	public static <T> PageResult<T> of(IPage<T> iPage) {
        PageResult<T> result = new PageResult<>();
        result.setResult(iPage.getRecords());
        result.setPageNumber(iPage.getCurrent());
        result.setPageSize(iPage.getPages());
        result.setTotalCount(iPage.getTotal());
        result.setPageCount(iPage.getPages());
        return result;
	}
	
	
	 /**
     * 开始分页
     * 
     * @param list
     * @param pageNum
     *            页码
     * @param pageSize
     *            每页多少条数据
     * @return
     */
    public static <T> List<T> startPage(List<T> list, Integer pageNum, Integer pageSize) {
        if (list == null) {
            return null;
        }
        if (list.size() == 0) {
            return null;
        }

        Integer count = list.size(); // 记录总数
        Integer pageCount = pageCount(list, pageNum, pageSize);

        int fromIndex = 0; // 开始索引
        int toIndex = 0; // 结束索引

        if (pageCount < pageNum) {
            return null;
        }
        if (pageCount - pageNum != 0) {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = fromIndex + pageSize;
        } else {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = count;
        }

        List<T> pageList = list.subList(fromIndex, toIndex);

        return pageList;
    }

    public static <T> Integer pageCount(List<T> list, Integer pageNum, Integer pageSize) {
        if (list == null) {
            return 0;
        }
        if (list.size() == 0) {
            return 0;
        }
        Integer count = list.size(); // 记录总数
        Integer pageCount = 0; // 页数
        if (count % pageSize == 0) {
            pageCount = count / pageSize;
        } else {
            pageCount = count / pageSize + 1;
        }

        return pageCount;
    }
    
	public static <T> PageResult<T> emptyPage(){
    	PageResult<T> page=new PageResult<>();
    	page.setPageCount(0);
    	page.setPageNumber(1);
    	page.setPageSize(10);
    	page.setTotalCount(0);
    	page.setResult(new ArrayList<>());
    	return page;
    }
}
