package cn.com.qmth.mps.util;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import cn.com.qmth.mps.bean.User;


/**
 * servlet 工具
 *
 */
public class ServletUtil {

	/**
	 * 接口日志
	 */
	private static final Logger LOG = LoggerFactory.getLogger(ServletUtil.class);

	public static User getSessionUser() {
		ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder
				.getRequestAttributes();
		if (requestAttributes == null) {
			return null;
		}
		HttpServletRequest request = requestAttributes.getRequest();
		if (request == null) {
			return null;
		}
		User user = (User) ServletUtil.getRequest().getAttribute("accessEntity");
		if (null == user) {
			return null;
		}
		return user;
	}

	/**
	 * 获取request对象
	 *
	 * @return
	 */
	public static HttpServletRequest getRequest() {
		ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder
				.getRequestAttributes();
		if (requestAttributes == null) {
			return null;
		}
		HttpServletRequest request = requestAttributes.getRequest();
		return request;
	}

	/**
	 * 获取response对象
	 *
	 * @return
	 */
	public static HttpServletResponse getResponse() {
		ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder
				.getRequestAttributes();
		HttpServletResponse response = requestAttributes.getResponse();
		return response;
	}

	/**
	 * 输出响应流
	 *
	 * @param response
	 * @author
	 */
	public static void returnJson(Object body, HttpServletResponse response) {
		response.setContentType("application/json;charset=utf-8");
		PrintWriter writer = null;
		try {
			writer = response.getWriter();
			String json = "{}";
			if (null != body) {
				json = JsonUtil.toJson(body);
			}
			writer.write(json);

			if (LOG.isDebugEnabled()) {
				LOG.debug("responseMsg = " + json);
			}
		} catch (IOException e) {
			LOG.error("returnJson fail... " + e.getMessage(), e);
		} finally {
			if (writer != null) {
				writer.close();
			}
		}
	}

	/**
	 * 输出响应流(无日志)
	 *
	 * @param response
	 * @author
	 */
	public static void returnJsonWithoutLog(Object body, HttpServletResponse response) {
		response.setContentType("application/json;charset=utf-8");
		PrintWriter writer = null;
		try {
			writer = response.getWriter();
			String json = "{}";
			if (null != body) {
				json = JsonUtil.toJson(body);
			}
			writer.write(json);
		} catch (IOException e) {
			// ignore
		} finally {
			if (writer != null) {
				writer.close();
			}
		}
	}

}
