package cn.com.qmth.mps.util;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.google.common.collect.Maps;
import com.qmth.boot.tools.uuid.FastUUID;

/**
 * 线程本地化工具
 *
 * @author 
 */
public class ThreadLocalUtil {
	/**
	 * trace ID
	 */
	private static final ThreadLocal<String> LOCAL_TRACE_ID = new ThreadLocal<String>() {
		@Override
		public String initialValue() {
			return null;
		}
	};

	/**
	 * map
	 */
	private static final ThreadLocal<Map<String, Object>> LOCAL_TRACE_MAP = new ThreadLocal<Map<String, Object>>() {
		@Override
		public Map<String, Object> initialValue() {
			return null;
		}
	};

	/**
	 * 重新初始化
	 *
	 * @author 
	 * @return
	 */
	public static String next() {
		LOCAL_TRACE_MAP.set(null);
		String traceID = FastUUID.get();
		setTraceId(traceID);
		return traceID;
	}

	/**
	 * 获取 trace ID
	 *
	 * @author 
	 * @return
	 */
	public static String getTraceId() {
		String traceID = LOCAL_TRACE_ID.get();
		if (traceID == null) {
			traceID = next();
		} else {
			return traceID;
		}

		return traceID;
	}

	/**
	 * 设置 trace ID
	 *
	 * @author 
	 * @param traceId
	 */
	public static void setTraceId(String traceId) {
		if (!(StringUtils.isNotBlank(traceId)))
			return;
		LOCAL_TRACE_ID.set(traceId);
	}

	/**
	 * 设置属性
	 *
	 * @author 
	 * @param key
	 * @param value
	 */
	public static void set(String key, Object value) {
		Map<String, Object> map = LOCAL_TRACE_MAP.get();

		if (null == map) {
			map = Maps.newHashMap();
			LOCAL_TRACE_MAP.set(map);
		}
		map.put(key, value);
	}

	/**
	 * 获取属性
	 *
	 * @author 
	 * @param key
	 * @return
	 */
	public static Object get(String key) {
		Map<String, Object> map = LOCAL_TRACE_MAP.get();
		if (null == map) {
			return null;
		}
		return map.get(key);
	}

	/**
	 * 清理属性
	 *
	 * @author 
	 */
	public static void clearAll() {
		Map<String, Object> map = LOCAL_TRACE_MAP.get();
		if (null != map) {
			map.clear();
		}
	}
}